<?php
/**
 * PHPUnit
 *
 * Copyright (c) 2002-2008, Sebastian Bergmann <sb@sebastian-bergmann.de>.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *   * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *   * Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 *
 *   * Neither the name of Sebastian Bergmann nor the names of his
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 * ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * @category   Testing
 * @package    PHPUnit
 * @author     Sebastian Bergmann <sb@sebastian-bergmann.de>
 * @copyright  2002-2008 Sebastian Bergmann <sb@sebastian-bergmann.de>
 * @license    http://www.opensource.org/licenses/bsd-license.php  BSD License
 * @version    SVN: $Id: XML.php 3165 2008-06-08 12:23:59Z sb $
 * @link       http://www.phpunit.de/
 * @since      File available since Release 2.3.0
 */

require_once 'PHPUnit/Framework.php';
require_once 'PHPUnit/Util/Class.php';
require_once 'PHPUnit/Util/Filter.php';
require_once 'PHPUnit/Util/Printer.php';
require_once 'PHPUnit/Util/XML.php';

PHPUnit_Util_Filter::addFileToFilter(__FILE__, 'PHPUNIT');

/**
 * A TestListener that generates an XML-based logfile
 * of the test execution.
 *
 * @category   Testing
 * @package    PHPUnit
 * @author     Sebastian Bergmann <sb@sebastian-bergmann.de>
 * @copyright  2002-2008 Sebastian Bergmann <sb@sebastian-bergmann.de>
 * @license    http://www.opensource.org/licenses/bsd-license.php  BSD License
 * @version    Release: 3.2.21
 * @link       http://www.phpunit.de/
 * @since      Class available since Release 2.1.0
 */
class PHPUnit_Util_Log_JUnit extends PHPUnit_Util_Printer implements PHPUnit_Framework_TestListener
{
    /**
     * @var    DOMDocument
     */
    protected $document;

    /**
     * @var    DOMElement
     */
    protected $root;

    /**
     * @var    boolean
     */
    protected $logIncompleteSkipped = FALSE;

    /**
     * @var    boolean
     */
    protected $writeDocument = TRUE;
    
    /**
     * @var    boolean
     */
    protected $addTestSuiteAttribute = TRUE;

    /**
     * @var    integer
     */
    protected $testSuiteTests = 0;

    /**
     * @var    integer
     */
    protected $testSuiteErrors = 0;

    /**
     * @var    integer
     */
    protected $testSuiteFailures = 0;

    /**
     * @var    integer
     */
    protected $testSuiteTimes = 0;



    /**
     * @var    DOMElement
     */
    protected $currentTestCase = NULL;


    /**
     * Constructor.
     *
     * @param  mixed   $out
     * @param  boolean $logIncompleteSkipped
     */
    public function __construct($out = NULL, $logIncompleteSkipped = FALSE)
    {
        $this->document = new DOMDocument('1.0', 'UTF-8');
        $this->document->formatOutput = TRUE;

        $this->root = $this->document->createElement('testsuite');
        $this->document->appendChild($this->root);

        parent::__construct($out);

        $this->logIncompleteSkipped = $logIncompleteSkipped;
    }

    /**
     * Flush buffer and close output.
     *
     */
    public function flush()
    {
        if ($this->writeDocument === TRUE) {
            $this->write($this->getXML());
        }

        parent::flush();
    }

    /**
     * An error occurred.
     *
     * @param  PHPUnit_Framework_Test $test
     * @param  Exception              $e
     * @param  float                  $time
     */
    public function addError(PHPUnit_Framework_Test $test, Exception $e, $time)
    {
        if (!$test instanceof PHPUnit_Framework_Warning) 
        { 
            $this->testSuiteErrors++;
                    
            // -----------------------------------------
            //                 Add error
            // -----------------------------------------
            $buffer = "";
            
            $error = $this->document->createElement('error');
            $error->setAttribute('type', get_class($e));  
            $error->setAttribute('message',$e->getMessage());
            
            if ($test instanceof PHPUnit_Framework_SelfDescribing) {
                $buffer = $test->toString() . "\n";
            } else {
                $buffer = '';
            }

            $buffer .= PHPUnit_Framework_TestFailure::exceptionToString($e) . "\n" .
                       PHPUnit_Util_Filter::getFilteredStacktrace($e, FALSE);

            $error->appendChild(
              $this->document->createCDATASection(
                PHPUnit_Util_XML::convertToUtf8($buffer)
              )
            );

            $this->currentTestCase->appendChild($error);
        }
    }

    /**
     * A failure occurred.
     *
     * @param  PHPUnit_Framework_Test                 $test
     * @param  PHPUnit_Framework_AssertionFailedError $e
     * @param  float                                  $time
     */
    public function addFailure(PHPUnit_Framework_Test $test, PHPUnit_Framework_AssertionFailedError $e, $time)
    {
        if (!$test instanceof PHPUnit_Framework_Warning) 
        {        
            $this->testSuiteFailures++; 
            
            // -----------------------------------------
            //                 Add error
            // -----------------------------------------
            $buffer = "";
            
            $error = $this->document->createElement('failure');
            $error->setAttribute('type', get_class($e));  
            $error->setAttribute('message',$e->getMessage());
            
            
            if ($test instanceof PHPUnit_Framework_SelfDescribing) {
                $buffer = $test->toString() . "\n";
            } else {
                $buffer = '';
            } 

            $buffer .= PHPUnit_Framework_TestFailure::exceptionToString($e) . "\n" .
                       PHPUnit_Util_Filter::getFilteredStacktrace($e, FALSE);
            
            $error->appendChild(
              $this->document->createCDATASection(
                PHPUnit_Util_XML::convertToUtf8($buffer)
              )
            );

            $this->currentTestCase->appendChild($error);
        }
    }

    /**
     * Incomplete test.
     *
     * @param  PHPUnit_Framework_Test $test
     * @param  Exception              $e
     * @param  float                  $time
     */
    public function addIncompleteTest(PHPUnit_Framework_Test $test, Exception $e, $time)
    {
        if ($this->logIncompleteSkipped) 
        { 
            $this->addError($test,$e, $time);
        }
    }

    /**
     * Skipped test.
     *
     * @param  PHPUnit_Framework_Test $test
     * @param  Exception              $e
     * @param  float                  $time
     * @since  Method available since Release 3.0.0
     */
    public function addSkippedTest(PHPUnit_Framework_Test $test, Exception $e, $time)
    {
        if ($this->logIncompleteSkipped) 
        { 
            $this->addError($test,$e, $time);
        }
    }

    
    /**
     * A testsuite started.
     *
     * @param  PHPUnit_Framework_TestSuite $suite
     * @since  Method available since Release 2.2.0
     */
    public function startTestSuite(PHPUnit_Framework_TestSuite $suite)
    {
        // Vide
    }

    /**
     * A testsuite ended.
     *
     * @param  PHPUnit_Framework_TestSuite $suite
     * @since  Method available since Release 2.2.0
     */
    public function endTestSuite(PHPUnit_Framework_TestSuite $suite)
    {
        // Vide      
    }

    /**
     * A test started.
     *
     * @param  PHPUnit_Framework_Test $test
     */
    public function startTest(PHPUnit_Framework_Test $test)
    {
        if (!$test instanceof PHPUnit_Framework_Warning) {
            
            
            $testCase = $this->document->createElement('testcase');            
            $testCase->setAttribute('name', $test->getName());

            if ($test instanceof PHPUnit_Framework_TestCase) {
                $class      = new ReflectionClass($test);
                $methodName = $test->getName();

                if ($class->hasMethod($methodName)) {
                    $method = $class->getMethod($test->getName());

                    $strClassName = $class->getFileName();
                    $strClassName = substr(strstr($strClassName,'test\\'), 5);            
                    $strClassName = str_replace('\\','.',$strClassName);   
                    $strClassName = str_replace('.php','',$strClassName); 
                    
                    //$testCase->setAttribute('classname', $class->getName()); 
                    $testCase->setAttribute('classname', $strClassName);                    
                }
                else
                {
                    $testCase->setAttribute('classname', $class->getName());   
                }
            }
            else
            {
                $testCase->setAttribute('classname', $test->getName()); 
            }

            $this->currentTestCase = $testCase;
        }
    }

    /**
     * A test ended.
     *
     * @param  PHPUnit_Framework_Test $test
     * @param  float                  $time
     */
    public function endTest(PHPUnit_Framework_Test $test, $time)
    {
        if (!$test instanceof PHPUnit_Framework_Warning) {

            $this->currentTestCase->setAttribute('time', sprintf('%F', $time));

            $this->testSuiteTests++;
            $this->testSuiteTimes += $time;
            
            $this->root->appendChild($this->currentTestCase);
        }

        $this->currentTestCase       = NULL;
    }

    /**
     * Returns the XML as a string.
     *
     * @return string
     * @since  Method available since Release 2.2.0
     */
    public function getXML()
    {
        if ($this->addTestSuiteAttribute)
        {
            $hostname = gethostbyaddr($_SERVER['REMOTE_ADDR']);

            $this->root->setAttribute("errors", $this->testSuiteErrors);
            $this->root->setAttribute("failures", $this->testSuiteFailures);
            $this->root->setAttribute("hostname", $hostname);
            $this->root->setAttribute("name", "AllTests");
            $this->root->setAttribute("tests", $this->testSuiteTests); 
            $this->root->setAttribute("time", sprintf('%F', $this->testSuiteTimes));  
            $this->root->setAttribute("timestamp", date('Y-m-d\TH:i:s'));             
            
            $this->addTestSuiteAttribute = false;
        }
        
        return $this->document->saveXML();
    }

    /**
     * Enables or disables the writing of the document
     * in flush().
     *
     * This is a "hack" needed for the integration of
     * PHPUnit with Phing.
     *
     * @return string
     * @since  Method available since Release 2.2.0
     */
    public function setWriteDocument($flag)
    {
        if (is_bool($flag)) {
            $this->writeDocument = $flag;
        }
    }
}
?>
