<?php
/**
* Constructs the WFSProxy instance according to the type of request ...
* Construit le bon type de feature server proxy selon le type de requete
*
* PHP versions 5
* 
* @category  PHP
* @package   GeoPrisma  
* @author    Etienne Dube
* @copyright 2011, Boreal - Information Strategies
* @license   http://www.geoprisma.org/license BSD License
* @link      http://www.geoprisma.org
*/

/**
* WFS proxy factory
* 
* @category   PHP 
* @package    GeoPrisma
* @subpackage Proxy 
* @author     Etienne Dube
*/  
class org_geoprisma_proxy_wfs_WFSProxyFactory
{
    
    const WFS_OP_GETCAPABILITIES = 1;
    const WFS_OP_DESCRIBEFEATURETYPE = 2;
    const WFS_OP_GETFEATURE = 3;
    const WFS_OP_TRANSACTION = 4;
    
    const WFS_TRANSACTION_INSERT = 1;
    const WFS_TRANSACTION_UPDATE = 2;
    const WFS_TRANSACTION_DELETE = 3;

    /**
    * Returns the right kind of feature server proxy
    * 
    * @param org_geoprisma_service_WFSService $pobjService Service
    * 
    * @return org_geoprisma_proxy_wfs_WFSGetCapabilityProxy
    */
    public static function getWFSProxy(org_geoprisma_service_WFSService $pobjService)
    {

        $strPostRequest = null;
        $strContentType = null;
        $iRequestType = null;
        $iOPType = null;
        
        // since WFS requests can be done as POST calls with an XML document as the request,
        // we have to retrieve the contents of this document once from the input stream
        if ( $_SERVER['REQUEST_METHOD'] == 'POST' )
        {
            if (isset($_REQUEST) && isset($_REQUEST['data']))
            {
                $strPostRequest = $_REQUEST['data'];
                $iRequestType = org_geoprisma_proxy_wfs_WFSProxy::REQUEST_TYPE_POSTXML;
            }
            else
            {
                $strPostRequest = file_get_contents('php://input');
                // is this a XML or form/urlencoded request?
                $strContentType = $_SERVER['CONTENT_TYPE'];
                if ($strContentType != null)
                {
                    $strTOk = strtok($strContentType, ';');
                    // OGC spec says that content type MUST be set to text/xml in case of an XML request,
                    // but OpenLayers WFS protocol seems to use application/xml instead...
                    if ($strTOk && ($strTOk == 'text/xml' || $strTOk == 'application/xml') )
                    {
                        $iRequestType = org_geoprisma_proxy_wfs_WFSProxy::REQUEST_TYPE_POSTXML;
                    }
                    else
                    {
                        $iRequestType = org_geoprisma_proxy_wfs_WFSProxy::REQUEST_TYPE_POST;
                    }
                }
            }
        }
        else {
            $iRequestType = org_geoprisma_proxy_wfs_WFSProxy::REQUEST_TYPE_GET;
        }
        
        if ($iRequestType == org_geoprisma_proxy_wfs_WFSProxy::REQUEST_TYPE_POSTXML)
        {
            $iOPType = self::getOperationFromXml($strPostRequest);
        }
        
        if ($iRequestType == org_geoprisma_proxy_wfs_WFSProxy::REQUEST_TYPE_GET)
        {
            $iOPType = self::getOperationFromGET($strPostRequest);
        }


        // Set default to read
        $objWFSProxy = null;//new org_geoprisma_proxy_wfs_WFSReadProxy($pobjService);
        switch ($iOPType)
        {

            case self::WFS_OP_GETCAPABILITIES:
                $objWFSProxy = new org_geoprisma_proxy_wfs_WFSReadProxy($pobjService, $strContentType, $iRequestType, $strPostRequest);
                break;
            case self::WFS_OP_DESCRIBEFEATURETYPE:
                $objWFSProxy = new org_geoprisma_proxy_wfs_WFSReadProxy($pobjService, $strContentType, $iRequestType, $strPostRequest);
                break;
            case self::WFS_OP_GETFEATURE:
                $objWFSProxy = new org_geoprisma_proxy_wfs_WFSReadProxy($pobjService, $strContentType, $iRequestType, $strPostRequest);
                break;
            case self::WFS_OP_TRANSACTION:
                break;
        }

        if ($objWFSProxy == null)
        {
            throw new com_borealis_foundation_exception_AssertException("Proxy method not handled.");
        }

        return $objWFSProxy;
    }

    /**
    * Returns the type of operation from GET request
    * 
    * @return integer
    */
    private static function getOperationFromGET()
    {
        $strRequest = '';

        foreach ($_GET as $strKey => $strValue)
        {
            if (strtoupper($strKey) == 'REQUEST')
            {
                $strRequest = $strValue;
                break;
            }
        }

        switch ($strRequest)
        {
            case 'GetCapabilities':
                return self::WFS_OP_GETCAPABILITIES;
                break;
            case 'DescribeFeatureType':
                return self::WFS_OP_DESCRIBEFEATURETYPE;
                break;
            case 'GetFeature':
                return self::WFS_OP_GETFEATURE;
                break;
        }
        
        // unknown request type
        return null;
    }
    
    /**
    * Returns the type of operation from xml request
    * 
    * @param string $pstrXMLRequest XML request string
    *
    * @return integer
    */
    private static function getOperationFromXml($pstrXMLRequest)
    {
        $objDomDoc = new DOMDocument();
        if ($objDomDoc->loadXML($pstrXMLRequest))
        {
            $objSimpleXMLElement = simplexml_import_dom($objDomDoc);
            switch ( $objSimpleXMLElement->getName() )
            {
                case 'GetCapabilities':
                    return self::WFS_OP_GETCAPABILITIES;
                    break;
                case 'DescribeFeatureType':
                    return self::WFS_OP_DESCRIBEFEATURETYPE;
                    break;
                case 'GetFeature':
                    return self::WFS_OP_GETFEATURE;
                    break;
            }
        }
        
        // unknown request type
        return null;
    }
    
}
?>
