<?php
/**
* Classe de base des ressource, Lier aux fichier de configuration  
*
* PHP versions 5
* 
* @category  PHP
* @package   GeoPrisma  
* @author    Pascal Martin 
* @copyright 2009, Boreal - Information Strategies
* @license   http://www.geoprisma.org/license BSD License
* @link      http://www.geoprisma.org
*/

/**
* Classe de base des ressource, Lier aux fichier de configuration 
* 
* @category   PHP 
* @package    GeoPrisma
* @subpackage Resource 
* @author     Pascal Martin 
*/ 
class org_geoprisma_resource_Resource extends org_geoprisma_ConfigElement
{
    private $m_strName = null;
    private $m_objArrayDatastore = null;
    private $m_objArrayField = null;
    private $m_objArrayWidget = null;
    private $m_objArrayAccessFilter = null;
    private $m_objArrayOption = null;  
    private $m_strTitle = null;
    private $m_strACLName = null;
    
    /**
    * Construit un instance de org_geoprisma_resource_Resource
    * 
    * @param string      $pstrName              Resource name
    * @param ArrayObject $pobjArrayDatastore    Liste de datastore lier
    * @param ArrayObject $pobjArrayWidget       Liste de widget lier
    * @param ArrayObject $pobjArrayAccessFilter AccessFilter list
    * @param ArrayObject $pobjArrayOption       Liste des options du widget
    * @param string      $pstrTitle             Resource title
    * @param ArrayObject $pobjArrayField        Fields array object (optional)
    * @param string      $pstrACLName           Resource alternate name for the
    *                                           acl (optional)
    * 
    * @return org_geoprisma_resource_Resource
    */
    public function __construct(
        $pstrName, 
        ArrayObject $pobjArrayDatastore, 
        ArrayObject $pobjArrayWidget, 
        ArrayObject $pobjArrayAccessFilter, 
        ArrayObject $pobjArrayOption,
        $pstrTitle=null,
        $pobjArrayField=null,
        $pstrACLName=""
    ) {
        com_borealis_foundation_util_Assert::assertIsNotEmptyString($pstrName);
        $this->m_strName = $pstrName;
        $this->m_objArrayDatastore = $pobjArrayDatastore;
        $this->m_objArrayWidget = $pobjArrayWidget;
        $this->m_objArrayAccessFilter = $pobjArrayAccessFilter;
        $this->m_objArrayOption = $pobjArrayOption;
        $this->m_strTitle = ($pstrTitle != null) ? $pstrTitle : $pstrName;
        $this->m_objArrayField = ($pobjArrayField != null)
            ? $pobjArrayField
            : new ArrayObject();
        $this->m_strACLName = $pstrACLName;
    }

    /**
    * Resource name
    *
    * @return string
    */
    public function getName()
    {
        return $this->m_strName;   
    }

    /**
    * Resource title
    *
    * @return string
    */
    public function getTitle()
    {
        return $this->m_strTitle;   
    }

    /**
    * Resource alternate name for the acl
    *
    * @return string
    */
    public function getACLName()
    {
        return $this->m_strACLName;   
    }
    
    /**
    * Ressource Datastores
    *
    * @return ArrayObject
    */
    public function getDatastores()
    {
        return $this->m_objArrayDatastore;   
    }

    /**
    * Resource fields
    *
    * @return ArrayObject
    */
    public function getFields()
    {
        return $this->m_objArrayField;   
    }

    /**
    * Get a field using a property value ('name' by default).
    * 
    * @param string  $pstrValue    Option value
    * @param string  $pstrProperty Name of the property to search
    * @param boolean $pbSkipError  Wheter the method should skip throwing an
    *                              error if no field is found.
    * 
    * @return org_geoprisma_field_Field
    */
    public function getField(
        $pstrValue, $pstrProperty="name", $pbSkipError=false
    ) {
        if ($this->getFields() != null) 
        {
            foreach ($this->getFields() as $objField)   
            {
                switch ($pstrProperty) 
                {
                    case "name":
                        if ($objField->getName() == $pstrValue) 
                        {
                            return $objField;
                        }  
                        break;
                    case "id":
                        if ($objField->getId() == $pstrValue) 
                        {
                            return $objField;
                        }  
                        break;
                    default:
                        return false;
                        break;
                }
  
            }
        }

        if ($pbSkipError) 
        {
            return false;
        }
        else
        {
            throw new org_geoprisma_exception_FieldNotFoundException($pstrValue);
        }
    }
    
    /**
    * Ressource Widgets
    *
    * @return string
    */
    public function getWidgets()
    {
        return $this->m_objArrayWidget;   
    }

    /**
    * Add widget to array
    *
    * @param org_geoprisma_widget_Widget $pobjWidget Widget object to add.
    *
    * @return void
    */
    public function addWidget($pobjWidget) 
    {
        $this->m_objArrayWidget->append($pobjWidget);
    }
    
    /**
    * Ressource AccessFilters
    *
    * @return string
    */
    public function getAccessFilters()
    {
        return $this->m_objArrayAccessFilter;   
    }

    /**
    * Add accessfilter to array
    *
    * @param org_geoprisma_widget_AccessFilter $pobjAccessFilter AccessFilter object to add.
    *
    * @return void
    */
    public function addAccessFilter($pobjAccessFilter) 
    {
        $this->m_objArrayAccessFilter->append($pobjAccessFilter);
    }
    
    /**
    * Ressource Options
    *
    * @return ArrayObject
    */
    public function getOptions()
    {
        return $this->m_objArrayOption;   
    }
    
    /**
    * Retourne l'option demander
    * 
    * @param string  $pstrName    Option name
    * @param boolean $pbSkipError Wheter the method should skip throwing an
    *                             error if no option is found.
    * 
    * @return org_geoprisma_option_Option
    */
    public function getOption($pstrName, $pbSkipError=false)
    {
        if ($this->getOptions() != null) 
        {
            foreach ($this->getOptions() as $objOption)   
            {
                if ($objOption->getName() == $pstrName) 
                {
                    return $objOption;
                }    
            }
        }

        if ($pbSkipError) 
        {
            return false;
        }
        else
        {
            throw new org_geoprisma_exception_OptionNotFoundException($pstrName);
        }
    }
    
    /**
    * Find the datastore corresponding to the service
    * 
    * @param org_geoprisma_service_Service $pobjService Service
    * 
    * @return org_geoprisma_datastore_Datastore
    */
    public function getDatastore(org_geoprisma_service_Service $pobjService)
    {
        foreach ($this->getDatastores() as $objDatastore)
        {
            if ($objDatastore->getService() == $pobjService) 
            {
                return $objDatastore; 
            }  
        }
        return null;   
    }

    /**
    * Find the datastore corresponding to a service type.
    * 
    * @param string $pstrServiceType Service type
    * 
    * @return org_geoprisma_datastore_Datastore
    */
    public function getDatastoreByServiceType($pstrServiceType)
    {
        foreach ($this->getDatastores() as $objDatastore)
        {
            if ($objDatastore->getService()->getType() == $pstrServiceType) 
            {
                return $objDatastore; 
            }  
        }
        return null;   
    }
    
    /**
    *  Ajout la resource courant (this) dans le xml      
    * 
    * @param DOMElement  &$pobjDomElementResources Noeud xml ou le resource doit être ajouter
    * @param DOMDocument &$pobjDomDocument         Document XMl ou le noeud xml datastore est ajouter, utiliser pour construire les noeuds
    * 
    * @return void
    */
    public function insertIntoXML(
        DOMElement &$pobjDomElementResources, DOMDocument &$pobjDomDocument
    ) {

        $objDomElementResource = $pobjDomDocument->createElement(
            'resource'
        );
        $pobjDomElementResources->appendChild($objDomElementResource);

        $objDomElementName = $pobjDomDocument->createElement(
            'name', $this->getName()
        ); 
        $objDomElementResource->appendChild($objDomElementName);

        $objDomElementTitle = $this->getDomElements(
            'title', $this->getTitle(), $pobjDomDocument
        );
        $objDomElementResource->appendChild($objDomElementTitle);

        // datastores
        $objDomElementDatastores = $pobjDomDocument->createElement(
            'datastores'
        );
        $objDomElementResource->appendChild($objDomElementDatastores);
        foreach ($this->getDatastores() as $objDatastore) 
        {
            $objDomElementDatastore = $pobjDomDocument->createElement(
                'datastore', $objDatastore->getName()
            );
            $objDomElementDatastores->appendChild(
                $objDomElementDatastore
            );
        }

        // fields
        $objDomElementFields = $pobjDomDocument->createElement(
            'fields'
        );
        $objDomElementResource->appendChild($objDomElementFields);
        foreach ($this->getFields() as $objField) 
        {
            $objField->insertIntoXML($objDomElementFields, $pobjDomDocument);
        }


        // create <options> node with its childs and append to 
        // <resource>
        $objDomElementResource->appendChild(
            $this->getDomElements(
                'options', $this->getOptions(), $pobjDomDocument
            )
        );
    }
    
    /**
    * Checks among its widgets if there is at least one using a given service
    *     type and (optionnaly) if it needs to interact with a layer (of 
    *     assumed same service type).
    * 
    * @param string  $pstrServiceType  Service type
    * @param boolean $pbCheckNeedLayer Wether the method should also check if
    *                                  the widget needs a layer as well.
    * 
    * @return boolean
    */
    public function hasWidgetUsingServiceType(
        $pstrServiceType, $pbCheckNeedLayer=true
    ) {
        $bResourceHasWidgetUsingServiceType = false;

        foreach ($this->getWidgets() as $objWidget) 
        {
            if ($objWidget->getServiceType() == $pstrServiceType
                && (!$pbCheckNeedLayer || $objWidget->needLayer())
            ) 
            {
                $bResourceHasWidgetUsingServiceType = true;
                break;
            }
        }

        return $bResourceHasWidgetUsingServiceType;
    }

    /**
    * Get the resource WFS Datastore, then call the according WFS Service
    * "getDescribeFeatureType" method.
    * 
    * @return string
    */
    public function getWFSDescribeFeatureType()
    {
        $strResponse = "";
        $objWFSDataStore = $this->getDatastoreByServiceType("wfs");
        if ($objWFSDataStore) 
        {
            $strResponse
                = $objWFSDataStore->getService()->getDescribeFeatureType(
                    $objWFSDataStore->getLayerString()
                );
        }
        return $strResponse;
    }

    /**
    * Get the resource WMS Datastore, then call the according WMS Service
    * "getStyles" method.
    * 
    * @return string
    */
    public function getWMSStyles()
    {
        $strResponse = "";
        $objWMSDataStore = $this->getDatastoreByServiceType("wms");
        if ($objWMSDataStore) 
        {
            $strResponse = $objWMSDataStore->getService()->getStyles(
                $objWMSDataStore->getLayerString()
            );
        }
        return $strResponse;
    }

    /**
    * Get the resource according Datastore, then call the according Service
    * "getFeature" method.
    *
    * @param string $pstrServiceType Service type
    * @param string $pstrValue       Condition value for the request
    * 
    * @return string
    */
    public function getFeature($pstrServiceType, $pstrValue)
    {
        $strResponse = "";
        $objDataStore = $this->getDatastoreByServiceType($pstrServiceType);
        if ($objDataStore) 
        {
            $strResponse = $objDataStore->getService()->getFeature(
                $objDataStore->getLayerString(), $pstrValue
            );
        }
        return $strResponse;
    }
}

?>
